# -*- coding: utf-8 -*-


"""
pep8ify
~~~~~~~~

:copyright: (c) 2012 by Steve Pulec.

"""

__title__ = 'pep8ify'
__version__ = '0.0.5'
__author__ = 'Steve Pulec'
__license__ = 'Apache License 2.0'
__copyright__ = 'Copyright 2012 Steve Pulec'
